from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

dropbox_user_disabled2_fa_tests: list[RuleTest] = [
    RuleTest(
        name="2FA Disabled",
        expected_result=True,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
                },
            },
            "context": {
                "_tag": "team_member",
                "account_id": "dbid:AAAAAAAAAAAAAAAA",
                "display_name": "Alice Bob",
                "email": "alice.bob@company.io",
                "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
            },
            "details": {
                ".tag": "tfa_change_status_details",
                "new_value": {".tag": "disabled"},
                "previous_value": {".tag": "authenticator"},
                "used_rescue_code": True,
            },
            "event_category": {"_tag": "tfa"},
            "event_type": {
                "_tag": "tfa_change_status",
                "description": "Enabled/disabled/changed two-step verification setting",
            },
            "involve_non_team_member": False,
            "origin": {
                "access_method": {
                    ".tag": "end_user",
                    "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                },
                "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
            },
            "p_any_emails": ["alice.bob@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_usernames": ["Alice Bob"],
            "p_event_time": "2023-04-18 18:16:27",
            "p_log_type": "Dropbox.TeamEvent",
            "p_parse_time": "2023-04-18 18:18:46.808",
            "p_row_id": "0eb86fcfca9bb1cdce9defd217b8ac03",
            "p_schema_version": 0,
            "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
            "p_source_label": "dropbox1",
            "timestamp": "2023-04-18 18:16:27",
        },
    ),
    RuleTest(
        name="Other",
        expected_result=False,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "user_name",
                    "email": "user@domain.com",
                    "team_member_id": "dbmid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                },
            },
            "context": {"_tag": "team"},
            "details": {
                ".tag": "app_link_member_details",
                "app_info": {
                    ".tag": "member_linked_app",
                    "app_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "personal-dropbox-app-name",
                },
            },
            "event_category": {"_tag": "apps"},
            "event_type": {"_tag": "app_link_member", "description": "Linked app for member"},
            "involve_non_team_member": False,
            "origin": {
                "access_method": {".tag": "api", "request_id": "dbarod:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"},
                "geo_location": {
                    "city": "Los Angeles",
                    "country": "US",
                    "ip_address": "1.2.3.4",
                    "region": "California",
                },
            },
            "timestamp": "2023-02-16 20:39:34",
        },
    ),
]


@panther_managed
class DropboxUserDisabled2FA(Rule):
    default_description = "Dropbox user has disabled 2fa login"
    display_name = "Dropbox User Disabled 2FA"
    default_reference = "https://help.dropbox.com/account-access/enable-two-step-verification"
    default_severity = Severity.LOW
    log_types = [LogType.DROPBOX_TEAM_EVENT]
    id = "Dropbox.User.Disabled.2FA-prototype"
    tests = dropbox_user_disabled2_fa_tests

    def rule(self, event):
        return all(
            [
                deep_get(event, "details", ".tag", default="") == "tfa_change_status_details",
                deep_get(event, "details", "new_value", ".tag") == "disabled",
            ],
        )

    def title(self, event):
        actor = deep_get(event, "actor", "user", "email", default="<EMAIL_NOT_FOUND>")
        target = deep_get(event, "context", "email", default="<TARGET_NOT_FOUND>")
        return f"Dropbox: [{actor}] disabled 2FA for [{target}]."



from pypanther import LogType, Rule, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context


@panther_managed
class NotionPageSharedToWeb(Rule):
    id = "Notion.PageSharedToWeb-prototype"
    display_name = "Notion Page Published to Web"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Security", "Information Disclosure"]
    default_severity = Severity.LOW
    default_description = "A Notion User published a page to the web."
    default_runbook = "Potential information exposure - review the shared page and rectify if needed."
    default_reference = "https://www.notion.so/help/public-pages-and-web-publishing"
    # These event types correspond to users adding or editing the default role on a public page
    event_types = ("page.permissions.shared_to_public_role_added", "page.permissions.shared_to_public_role_updated")

    def rule(self, event):
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") in self.event_types

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        page_name = event.deep_get("event", "details", "page_name", default="<NO_PAGE_NAME_FOUND>")
        return f"Notion User [{user}] changed the status of page [{page_name}] to public."

    def alert_context(self, event):
        context = notion_alert_context(event)
        page_name = event.deep_get("event", "details", "page_name", default="<NO_PAGE_NAME_FOUND>")
        context["page_name"] = page_name
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_project_settings_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk Org Project Stop Monitor",
        expected_result=True,
        log={
            "content": {
                "origin": "github",
                "target": {"branch": "some-branch", "id": 222222222, "name": "repo-name", "owner": "github-org"},
                "targetFile": "go.mod",
                "type": "gomodules",
            },
            "created": "2023-03-30 15:38:18.58",
            "event": "org.project.stop_monitor",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "projectId": "05555555-8555-2333-5aaa-600000000000",
            "userId": "05555555-3333-4ddd-8ccc-75555555555",
        },
    ),
    RuleTest(
        name="Project Ignore Create",
        expected_result=True,
        log={
            "content": {
                "created": "2023-03-20T12:23:06.356Z",
                "ignorePath": "*",
                "ignoredBy": {"id": "05555555-3333-4ddd-8ccc-75555555555"},
                "issueId": "SNYK-JS-UNDICI-3323845",
                "reason": "dev dependency",
                "reasonType": "wont-fix",
            },
            "created": "2023-03-20 12:23:08.363",
            "event": "org.project.ignore.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "projectId": "05555555-8555-2333-5aaa-600000000000",
            "userId": "05555555-3333-4ddd-8ccc-75555555555",
        },
    ),
    RuleTest(
        name="Snyk Group SSO Membership sync",
        expected_result=False,
        log={
            "content": {},
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


@panther_managed
class SnykProjectSettings(Rule):
    id = "Snyk.Project.Settings-prototype"
    display_name = "Snyk Project Settings"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_reference = "https://docs.snyk.io/snyk-admin/introduction-to-snyk-projects/view-and-edit-project-settings"
    default_severity = Severity.MEDIUM
    default_description = "Detects when Snyk Project settings are changed\n"
    summary_attributes = ["event"]
    tests = snyk_project_settings_tests
    # The bodies of these actions are quite diverse.
    # When projects are added, the logged detail is the sourceOrgId.
    # org.project.stop_monitor is logged for individual files
    #   that are ignored.
    # AND the equivalent for licenses",
    ACTIONS = [
        "org.sast_settings.edit",
        "org.project.attributes.edit",
        "org.project.add",
        "org.project.delete",
        "org.project.edit",
        "org.project.fix_pr.manual_open",
        "org.project.ignore.create",
        "org.project.ignore.delete",
        "org.project.ignore.edit",
        "org.project.monitor",
        "org.project.pr_check.edit",
        "org.project.remove",
        "org.project.settings.delete",
        "org.project.settings.edit",
        "org.project.stop_monitor",
        "org.license_rule.create",
        "org.license_rule.delete",
        "org.license_rule.edit",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            operation = ".".join(action.split(".")[1:]).title()
        return f"Snyk: [{group_or_org}] [{operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        # merge event in for the alert_context
        a_c.update(event)
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"

    def severity(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        if action == "org.project.fix_pr.manual_open":
            return "INFO"
        return "LOW"



import argparse

from pypanther import display, get_rule, list_rules, shared_args, upload
from pypanther.vendor.panther_analysis_tool import util


def setup_list_rules_parser(list_rules_parser: argparse.ArgumentParser):
    list_rules_parser.set_defaults(func=list_rules.run)
    shared_args.for_filtering(list_rules_parser)
    list_rules_parser.add_argument(
        "--managed",
        help="List panther managed rules",
        default=False,
        required=False,
        action="store_true",
    )
    list_rules_parser.add_argument(
        "--registered",
        help="List registered rules",
        default=False,
        required=False,
        action="store_true",
    )
    list_rules_parser.add_argument(
        "--attributes",
        help="Display attributes of rules as columns in printed table (i.e --attributes threshold default_display_name)",
        nargs="+",
        default=None,
        required=False,
        choices=display.VALID_RULE_TABLE_ATTRS,
    )
    list_rules_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.VALID_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )


def setup_get_rule_parser(get_rules_parser: argparse.ArgumentParser):
    get_rules_parser.set_defaults(func=get_rule.run)
    get_rules_parser.add_argument(
        "--id",
        help="Required. The id of the rule to get",
        required=True,
        type=str,
    )
    get_rules_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.VALID_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )


def setup_test_parser(test_parser: argparse.ArgumentParser):
    shared_args.for_filtering(test_parser)
    test_parser.add_argument(
        "--verbose",
        help="Verbose output, includes passing tests, skipped tests, and exception stack traces",
        default=False,
        required=False,
        action="store_true",
    )
    test_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.VALID_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )


def setup_upload_parser(upload_parser: argparse.ArgumentParser):
    upload_parser.set_defaults(func=util.func_with_backend(upload.run))
    upload_parser.add_argument(
        "--max-retries",
        help="Retry to upload on a failure for a maximum number of times",
        default=10,
        type=int,
        required=False,
    )
    upload_parser.add_argument(
        "--skip-tests",
        help="Skip running tests and go directly to upload",
        default=False,
        required=False,
        action="store_true",
    )
    upload_parser.add_argument(
        "--confirm",
        help="Proceed with the upload without requiring user input",
        default=False,
        required=False,
        action="store_true",
    )
    upload_parser.add_argument(
        "--verbose",
        help="Verbose output",
        default=False,
        required=False,
        action="store_true",
    )
    upload_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.VALID_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

box_new_login_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="New Login Event",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "ADD_LOGIN_ACTIVITY_DEVICE",
            "source": {"id": "12345678", "type": "user", "login": "user@example"},
        },
    ),
]


@panther_managed
class BoxNewLogin(Rule):
    id = "Box.New.Login-prototype"
    display_name = "Box New Login"
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Initial Access:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.INFO
    default_description = "A user logged in from a new device.\n"
    default_reference = "https://support.box.com/hc/en-us/articles/360043691914-Controlling-Devices-Used-to-Access-Box"
    default_runbook = "Investigate whether this is a valid user login.\n"
    summary_attributes = ["ip_address"]
    tests = box_new_login_tests

    def rule(self, event):
        # ADD_LOGIN_ACTIVITY_DEVICE
        #  detect when a user logs in from a device not previously seen
        return event.get("event_type") == "ADD_LOGIN_ACTIVITY_DEVICE"

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'name', default='<UNKNOWN_USER>')}] logged in from a new device."



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardCloudflareHttpReq(DataModel):
    id: str = "Standard.Cloudflare.HttpReq"
    display_name: str = "Cloudflare Firewall"
    enabled: bool = True
    log_types: list[str] = [LogType.CLOUDFLARE_HTTP_REQUEST]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="source_ip", path="ClientIP"),
        DataModelMapping(name="user_agent", path="ClientRequestUserAgent"),
        DataModelMapping(name="http_status", path="EdgeResponseStatus"),
        DataModelMapping(name="source_port", path="ClientSrcPort"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.helpers.panther_default import aws_cloudtrail_success
from pypanther.helpers.panther_iocs import XZ_AMIS

awsec2_vulnerable_xz_image_launched_tests: list[RuleTest] = [
    RuleTest(
        name="Single vulnerable AMI Launched",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "cd7919fe-34a2-4d26-b038-23a2556a79fb",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2024-04-02 15:13:06.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.09",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "439a7e66-d8b6-4bad-98d9-214c20161939",
            "requestParameters": {
                "blockDeviceMapping": {
                    "items": [
                        {
                            "deviceName": "/dev/sda1",
                            "ebs": {
                                "deleteOnTermination": True,
                                "encrypted": False,
                                "iops": 3000,
                                "snapshotId": "snap-00000000000000000",
                                "throughput": 125,
                                "volumeSize": 10,
                                "volumeType": "gp3",
                            },
                        },
                    ],
                },
                "clientToken": "00000000-0000-0000-0000-000000000000",
                "disableApiStop": False,
                "disableApiTermination": False,
                "ebsOptimized": True,
                "instanceType": "t3a.micro",
                "instancesSet": {
                    "items": [{"imageId": "ami-020a359780bc6f835", "keyName": "a-key", "maxCount": 1, "minCount": 1}],
                },
                "monitoring": {"enabled": False},
                "networkInterfaceSet": {
                    "items": [
                        {
                            "associatePublicIpAddress": True,
                            "deviceIndex": 0,
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000"}]},
                            "subnetId": "subnet-00000000000000000",
                        },
                    ],
                },
                "privateDnsNameOptions": {
                    "enableResourceNameDnsAAAARecord": False,
                    "enableResourceNameDnsARecord": False,
                    "hostnameType": "ip-name",
                },
                "tagSpecificationSet": {
                    "items": [{"resourceType": "instance", "tags": [{"key": "Name", "value": "test"}]}],
                },
            },
            "responseElements": {
                "groupSet": {},
                "instancesSet": {
                    "items": [
                        {
                            "amiLaunchIndex": 0,
                            "architecture": "x86_64",
                            "blockDeviceMapping": {},
                            "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                            "clientToken": "8cda61da-eea9-495c-b178-e7014d9bc212",
                            "cpuOptions": {"coreCount": 1, "threadsPerCore": 2},
                            "currentInstanceBootMode": "legacy-bios",
                            "ebsOptimized": True,
                            "enaSupport": True,
                            "enclaveOptions": {"enabled": False},
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}]},
                            "hypervisor": "xen",
                            "imageId": "ami-020a359780bc6f835",
                            "instanceId": "i-00000000000000000",
                            "instanceState": {"code": 0, "name": "pending"},
                            "instanceType": "t3a.micro",
                            "keyName": "a-key",
                            "launchTime": 1712070786000,
                            "maintenanceOptions": {"autoRecovery": "default"},
                            "metadataOptions": {
                                "httpEndpoint": "enabled",
                                "httpProtocolIpv4": "enabled",
                                "httpProtocolIpv6": "disabled",
                                "httpPutResponseHopLimit": 1,
                                "httpTokens": "optional",
                                "instanceMetadataTags": "disabled",
                                "state": "pending",
                            },
                            "monitoring": {"state": "disabled"},
                            "networkInterfaceSet": {
                                "items": [
                                    {
                                        "attachment": {
                                            "attachTime": 1712070786000,
                                            "attachmentId": "eni-attach-00000000000000000",
                                            "deleteOnTermination": True,
                                            "deviceIndex": 0,
                                            "networkCardIndex": 0,
                                            "status": "attaching",
                                        },
                                        "groupSet": {
                                            "items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}],
                                        },
                                        "interfaceType": "interface",
                                        "ipv6AddressesSet": {},
                                        "macAddress": "06:71:c7:76:de:4f",
                                        "networkInterfaceId": "eni-00000000000000000",
                                        "ownerId": "123456789012",
                                        "privateDnsName": "ip-10-0-0-3.us-west-2.compute.internal",
                                        "privateIpAddress": "10.0.0.3",
                                        "privateIpAddressesSet": {
                                            "item": [
                                                {
                                                    "primary": True,
                                                    "privateDnsName": "ip-10-0-0-3.us-west-2.compute.internal",
                                                    "privateIpAddress": "10.0.0.3",
                                                },
                                            ],
                                        },
                                        "sourceDestCheck": True,
                                        "status": "in-use",
                                        "subnetId": "subnet-00000000000000000",
                                        "tagSet": {},
                                        "vpcId": "vpc-00000000000000000",
                                    },
                                ],
                            },
                            "placement": {"availabilityZone": "us-west-2b", "tenancy": "default"},
                            "privateDnsName": "ip-10-0-0-3.us-west-2.compute.internal",
                            "privateDnsNameOptions": {
                                "enableResourceNameDnsAAAARecord": False,
                                "enableResourceNameDnsARecord": False,
                                "hostnameType": "ip-name",
                            },
                            "privateIpAddress": "10.0.0.3",
                            "productCodes": {},
                            "rootDeviceName": "/dev/sda1",
                            "rootDeviceType": "ebs",
                            "sourceDestCheck": True,
                            "stateReason": {"code": "pending", "message": "pending"},
                            "subnetId": "subnet-00000000000000000",
                            "tagSet": {"items": [{"key": "Name", "value": "test"}]},
                            "virtualizationType": "hvm",
                            "vpcId": "vpc-00000000000000000",
                        },
                    ],
                },
                "ownerId": "123456789012",
                "requestId": "439a7e66-d8b6-4bad-98d9-214c20161939",
                "reservationId": "r-00000000000000000",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "1.2.3.4",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:124.0) Gecko/20100101 Firefox/124.0",
            "userIdentity": {
                "accessKeyId": "ASIASXP6SDP2MXGX4MYC",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/user.name",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:user.name",
                "sessionContext": {
                    "attributes": {"creationDate": "2024-04-02T14:45:31Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "ARole",
                    },
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Multiple vulnerable AMIs Launched",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "762b9172-6148-4c76-aee4-f6fc0fd140af",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2024-04-02 16:46:24.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.09",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "a71ca9b3-be94-4e22-95c5-e93b2280306c",
            "requestParameters": {
                "blockDeviceMapping": {
                    "items": [
                        {
                            "deviceName": "/dev/sda1",
                            "ebs": {
                                "deleteOnTermination": True,
                                "encrypted": False,
                                "iops": 3000,
                                "snapshotId": "snap-00000000000000000",
                                "throughput": 125,
                                "volumeSize": 6,
                                "volumeType": "gp3",
                            },
                        },
                    ],
                },
                "clientToken": "8945829d-0ef6-470d-bbfd-ea5bb2d82fdb",
                "disableApiStop": False,
                "disableApiTermination": False,
                "ebsOptimized": True,
                "instanceType": "t3a.nano",
                "instancesSet": {
                    "items": [{"imageId": "ami-092e3b17e435e5e58", "keyName": "a-key", "maxCount": 2, "minCount": 2}],
                },
                "monitoring": {"enabled": False},
                "networkInterfaceSet": {
                    "items": [
                        {
                            "associatePublicIpAddress": True,
                            "deviceIndex": 0,
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000"}]},
                            "subnetId": "subnet-00000000000000000",
                        },
                    ],
                },
                "privateDnsNameOptions": {
                    "enableResourceNameDnsAAAARecord": False,
                    "enableResourceNameDnsARecord": False,
                    "hostnameType": "ip-name",
                },
                "tagSpecificationSet": {
                    "items": [{"resourceType": "instance", "tags": [{"key": "Name", "value": "test"}]}],
                },
            },
            "responseElements": {
                "groupSet": {},
                "instancesSet": {
                    "items": [
                        {
                            "amiLaunchIndex": 0,
                            "architecture": "x86_64",
                            "blockDeviceMapping": {},
                            "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                            "clientToken": "8945829d-0ef6-470d-bbfd-ea5bb2d82fdb",
                            "cpuOptions": {"coreCount": 1, "threadsPerCore": 2},
                            "currentInstanceBootMode": "legacy-bios",
                            "ebsOptimized": True,
                            "enaSupport": True,
                            "enclaveOptions": {"enabled": False},
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}]},
                            "hypervisor": "xen",
                            "imageId": "ami-092e3b17e435e5e58",
                            "instanceId": "i-00000000000000000",
                            "instanceState": {"code": 0, "name": "pending"},
                            "instanceType": "t3a.nano",
                            "keyName": "a-key",
                            "launchTime": 1712076384000,
                            "maintenanceOptions": {"autoRecovery": "default"},
                            "metadataOptions": {
                                "httpEndpoint": "enabled",
                                "httpProtocolIpv4": "enabled",
                                "httpProtocolIpv6": "disabled",
                                "httpPutResponseHopLimit": 1,
                                "httpTokens": "optional",
                                "instanceMetadataTags": "disabled",
                                "state": "pending",
                            },
                            "monitoring": {"state": "disabled"},
                            "networkInterfaceSet": {
                                "items": [
                                    {
                                        "attachment": {
                                            "attachTime": 1712076384000,
                                            "attachmentId": "eni-attach-00000000000000000",
                                            "deleteOnTermination": True,
                                            "deviceIndex": 0,
                                            "networkCardIndex": 0,
                                            "status": "attaching",
                                        },
                                        "groupSet": {
                                            "items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}],
                                        },
                                        "interfaceType": "interface",
                                        "ipv6AddressesSet": {},
                                        "macAddress": "06:18:9b:64:05:2b",
                                        "networkInterfaceId": "eni-00000000000000000",
                                        "ownerId": "123456789012",
                                        "privateDnsName": "ip-10-0-0-4.us-west-2.compute.internal",
                                        "privateIpAddress": "10.0.0.4",
                                        "privateIpAddressesSet": {
                                            "item": [
                                                {
                                                    "primary": True,
                                                    "privateDnsName": "ip-10-0-0-4.us-west-2.compute.internal",
                                                    "privateIpAddress": "10.0.0.4",
                                                },
                                            ],
                                        },
                                        "sourceDestCheck": True,
                                        "status": "in-use",
                                        "subnetId": "subnet-00000000000000000",
                                        "tagSet": {},
                                        "vpcId": "vpc-00000000000000000",
                                    },
                                ],
                            },
                            "placement": {"availabilityZone": "us-west-2b", "tenancy": "default"},
                            "privateDnsName": "ip-10-0-0-4.us-west-2.compute.internal",
                            "privateDnsNameOptions": {
                                "enableResourceNameDnsAAAARecord": False,
                                "enableResourceNameDnsARecord": False,
                                "hostnameType": "ip-name",
                            },
                            "privateIpAddress": "10.0.0.4",
                            "productCodes": {},
                            "rootDeviceName": "/dev/sda1",
                            "rootDeviceType": "ebs",
                            "sourceDestCheck": True,
                            "stateReason": {"code": "pending", "message": "pending"},
                            "subnetId": "subnet-00000000000000000",
                            "tagSet": {"items": [{"key": "Name", "value": "test"}]},
                            "virtualizationType": "hvm",
                            "vpcId": "vpc-00000000000000000",
                        },
                        {
                            "amiLaunchIndex": 1,
                            "architecture": "x86_64",
                            "blockDeviceMapping": {},
                            "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                            "clientToken": "8945829d-0ef6-470d-bbfd-ea5bb2d82fdb",
                            "cpuOptions": {"coreCount": 1, "threadsPerCore": 2},
                            "currentInstanceBootMode": "legacy-bios",
                            "ebsOptimized": True,
                            "enaSupport": True,
                            "enclaveOptions": {"enabled": False},
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}]},
                            "hypervisor": "xen",
                            "imageId": "ami-020a359780bc6f835",
                            "instanceId": "i-00000000000000001",
                            "instanceState": {"code": 0, "name": "pending"},
                            "instanceType": "t3a.nano",
                            "keyName": "a-key",
                            "launchTime": 1712076384000,
                            "maintenanceOptions": {"autoRecovery": "default"},
                            "metadataOptions": {
                                "httpEndpoint": "enabled",
                                "httpProtocolIpv4": "enabled",
                                "httpProtocolIpv6": "disabled",
                                "httpPutResponseHopLimit": 1,
                                "httpTokens": "optional",
                                "instanceMetadataTags": "disabled",
                                "state": "pending",
                            },
                            "monitoring": {"state": "disabled"},
                            "networkInterfaceSet": {
                                "items": [
                                    {
                                        "attachment": {
                                            "attachTime": 1712076384000,
                                            "attachmentId": "eni-attach-00000000000000000",
                                            "deleteOnTermination": True,
                                            "deviceIndex": 0,
                                            "networkCardIndex": 0,
                                            "status": "attaching",
                                        },
                                        "groupSet": {
                                            "items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}],
                                        },
                                        "interfaceType": "interface",
                                        "ipv6AddressesSet": {},
                                        "macAddress": "06:b3:32:47:a8:c5",
                                        "networkInterfaceId": "eni-00000000000000000",
                                        "ownerId": "123456789012",
                                        "privateDnsName": "ip-10-0-0-5.us-west-2.compute.internal",
                                        "privateIpAddress": "10.0.0.5",
                                        "privateIpAddressesSet": {
                                            "item": [
                                                {
                                                    "primary": True,
                                                    "privateDnsName": "ip-10-0-0-5.us-west-2.compute.internal",
                                                    "privateIpAddress": "10.0.0.5",
                                                },
                                            ],
                                        },
                                        "sourceDestCheck": True,
                                        "status": "in-use",
                                        "subnetId": "subnet-00000000000000000",
                                        "tagSet": {},
                                        "vpcId": "vpc-00000000000000000",
                                    },
                                ],
                            },
                            "placement": {"availabilityZone": "us-west-2b", "tenancy": "default"},
                            "privateDnsName": "ip-10-0-0-5.us-west-2.compute.internal",
                            "privateDnsNameOptions": {
                                "enableResourceNameDnsAAAARecord": False,
                                "enableResourceNameDnsARecord": False,
                                "hostnameType": "ip-name",
                            },
                            "privateIpAddress": "10.0.0.5",
                            "productCodes": {},
                            "rootDeviceName": "/dev/sda1",
                            "rootDeviceType": "ebs",
                            "sourceDestCheck": True,
                            "stateReason": {"code": "pending", "message": "pending"},
                            "subnetId": "subnet-00000000000000000",
                            "tagSet": {"items": [{"key": "Name", "value": "test"}]},
                            "virtualizationType": "hvm",
                            "vpcId": "vpc-00000000000000000",
                        },
                    ],
                },
                "ownerId": "123456789012",
                "requestId": "a71ca9b3-be94-4e22-95c5-e93b2280306c",
                "reservationId": "r-00000000000000000",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "1.2.3.4",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:124.0) Gecko/20100101 Firefox/124.0",
            "userIdentity": {
                "accessKeyId": "ASIASXP6SDP2MBQCDGHR",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/user.name",
                "principalId": "00000000000000000:user.name",
                "sessionContext": {
                    "attributes": {"creationDate": "2024-04-02T16:09:04Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "principalId": "00000000000000000",
                        "type": "Role",
                        "userName": "ARole",
                    },
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Non-vulnerable AMI Launched",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "cd7919fe-34a2-4d26-b038-23a2556a79fb",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2024-04-02 15:13:06.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.09",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "439a7e66-d8b6-4bad-98d9-214c20161939",
            "requestParameters": {
                "blockDeviceMapping": {
                    "items": [
                        {
                            "deviceName": "/dev/sda1",
                            "ebs": {
                                "deleteOnTermination": True,
                                "encrypted": False,
                                "iops": 3000,
                                "snapshotId": "snap-00000000000000000",
                                "throughput": 125,
                                "volumeSize": 10,
                                "volumeType": "gp3",
                            },
                        },
                    ],
                },
                "clientToken": "00000000-0000-0000-0000-000000000000",
                "disableApiStop": False,
                "disableApiTermination": False,
                "ebsOptimized": True,
                "instanceType": "t3a.micro",
                "instancesSet": {
                    "items": [{"imageId": "ami-08038de0f4f90a9f0", "keyName": "a-key", "maxCount": 1, "minCount": 1}],
                },
                "monitoring": {"enabled": False},
                "networkInterfaceSet": {
                    "items": [
                        {
                            "associatePublicIpAddress": True,
                            "deviceIndex": 0,
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000"}]},
                            "subnetId": "subnet-00000000000000000",
                        },
                    ],
                },
                "privateDnsNameOptions": {
                    "enableResourceNameDnsAAAARecord": False,
                    "enableResourceNameDnsARecord": False,
                    "hostnameType": "ip-name",
                },
                "tagSpecificationSet": {
                    "items": [{"resourceType": "instance", "tags": [{"key": "Name", "value": "test"}]}],
                },
            },
            "responseElements": {
                "groupSet": {},
                "instancesSet": {
                    "items": [
                        {
                            "amiLaunchIndex": 0,
                            "architecture": "x86_64",
                            "blockDeviceMapping": {},
                            "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                            "clientToken": "8cda61da-eea9-495c-b178-e7014d9bc212",
                            "cpuOptions": {"coreCount": 1, "threadsPerCore": 2},
                            "currentInstanceBootMode": "legacy-bios",
                            "ebsOptimized": True,
                            "enaSupport": True,
                            "enclaveOptions": {"enabled": False},
                            "groupSet": {"items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}]},
                            "hypervisor": "xen",
                            "imageId": "ami-08038de0f4f90a9f0",
                            "instanceId": "i-00000000000000000",
                            "instanceState": {"code": 0, "name": "pending"},
                            "instanceType": "t3a.micro",
                            "keyName": "a-key",
                            "launchTime": 1712070786000,
                            "maintenanceOptions": {"autoRecovery": "default"},
                            "metadataOptions": {
                                "httpEndpoint": "enabled",
                                "httpProtocolIpv4": "enabled",
                                "httpProtocolIpv6": "disabled",
                                "httpPutResponseHopLimit": 1,
                                "httpTokens": "optional",
                                "instanceMetadataTags": "disabled",
                                "state": "pending",
                            },
                            "monitoring": {"state": "disabled"},
                            "networkInterfaceSet": {
                                "items": [
                                    {
                                        "attachment": {
                                            "attachTime": 1712070786000,
                                            "attachmentId": "eni-attach-00000000000000000",
                                            "deleteOnTermination": True,
                                            "deviceIndex": 0,
                                            "networkCardIndex": 0,
                                            "status": "attaching",
                                        },
                                        "groupSet": {
                                            "items": [{"groupId": "sg-00000000000000000", "groupName": "ssh"}],
                                        },
                                        "interfaceType": "interface",
                                        "ipv6AddressesSet": {},
                                        "macAddress": "06:71:c7:76:de:4f",
                                        "networkInterfaceId": "eni-00000000000000000",
                                        "ownerId": "123456789012",
                                        "privateDnsName": "ip-10-0-0-3.us-west-2.compute.internal",
                                        "privateIpAddress": "10.0.0.3",
                                        "privateIpAddressesSet": {
                                            "item": [
                                                {
                                                    "primary": True,
                                                    "privateDnsName": "ip-10-0-0-3.us-west-2.compute.internal",
                                                    "privateIpAddress": "10.0.0.3",
                                                },
                                            ],
                                        },
                                        "sourceDestCheck": True,
                                        "status": "in-use",
                                        "subnetId": "subnet-00000000000000000",
                                        "tagSet": {},
                                        "vpcId": "vpc-00000000000000000",
                                    },
                                ],
                            },
                            "placement": {"availabilityZone": "us-west-2b", "tenancy": "default"},
                            "privateDnsName": "ip-10-0-0-3.us-west-2.compute.internal",
                            "privateDnsNameOptions": {
                                "enableResourceNameDnsAAAARecord": False,
                                "enableResourceNameDnsARecord": False,
                                "hostnameType": "ip-name",
                            },
                            "privateIpAddress": "10.0.0.3",
                            "productCodes": {},
                            "rootDeviceName": "/dev/sda1",
                            "rootDeviceType": "ebs",
                            "sourceDestCheck": True,
                            "stateReason": {"code": "pending", "message": "pending"},
                            "subnetId": "subnet-00000000000000000",
                            "tagSet": {"items": [{"key": "Name", "value": "test"}]},
                            "virtualizationType": "hvm",
                            "vpcId": "vpc-00000000000000000",
                        },
                    ],
                },
                "ownerId": "123456789012",
                "requestId": "439a7e66-d8b6-4bad-98d9-214c20161939",
                "reservationId": "r-00000000000000000",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "1.2.3.4",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:124.0) Gecko/20100101 Firefox/124.0",
            "userIdentity": {
                "accessKeyId": "ASIASXP6SDP2MXGX4MYC",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/ARole/user.name",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:user.name",
                "sessionContext": {
                    "attributes": {"creationDate": "2024-04-02T14:45:31Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/ARole",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "ARole",
                    },
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSEC2VulnerableXZImageLaunched(Rule):
    default_description = (
        "Detecting EC2 instances launched with AMIs containing potentially vulnerable versions of XZ (CVE-2024-3094)\n"
    )
    display_name = "AWS EC2 Vulnerable XZ Image Launched"
    default_reference = "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2024-3094"
    default_severity = Severity.CRITICAL
    tags = ["AWS", "Linux", "Emerging Threats", "Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195.001"]}
    default_runbook = "- Verify that the AMI is indeed vulnerable to CVE-2024-3094 (xz -V being 5.6.0 or 5.6.1) - If the AMI is vulnerable, terminate the instance and launch a new instance with a non-vulnerable AMI\n"
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.Vulnerable.XZ.Image.Launched-prototype"
    tests = awsec2_vulnerable_xz_image_launched_tests
    # AMIs published by Fedora between 2024-03-26 and 2024-04-02
    # OpenSUSE and Kali do not have any recent [public] AMIs that would be affected

    def rule(self, event):
        if not aws_cloudtrail_success(event) or event.get("eventName") != "RunInstances":
            return False
        amis_launched = event.deep_walk(
            "responseElements",
            "instancesSet",
            "items",
            "imageId",
            default="<AMI ID not found>",
            return_val="all",
        )
        # convert to a list if only one item is returned
        if not isinstance(amis_launched, list):
            amis_launched = [amis_launched]
        if any(ami in XZ_AMIS for ami in amis_launched):
            return True
        return False

    def title(self, event):
        amis_launched = event.deep_walk(
            "responseElements",
            "instancesSet",
            "items",
            "imageId",
            default="<AMI ID not found>",
            return_val="all",
        )
        instance_ids = event.deep_walk(
            "responseElements",
            "instancesSet",
            "items",
            "instanceId",
            default="<Instance ID not found>",
            return_val="all",
        )
        return f"Instance {instance_ids} launched with vulnerable AMI: {amis_launched}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

role_assumedby_user_tests: list[RuleTest] = [
    RuleTest(
        name="Role Assumed by Service",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "1f3d7d49-6637-3304-b959-9be15f20215d",
            "eventName": "AssumeRole",
            "eventSource": "sts.amazonaws.com",
            "eventTime": "2024-06-02 20:27:12",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "123456789123",
            "requestID": "a0dda101-6e27-4f88-8250-f3d475f88b56",
            "requestParameters": {
                "roleArn": "arn:aws:iam::123456789123:role/my_role_arn",
                "roleSessionName": "awslambda_55_20240602202712548",
            },
            "resources": [
                {
                    "accountId": "123456789123",
                    "arn": "arn:aws:iam::123456789123:role/my_role_arn",
                    "type": "AWS::IAM::Role",
                },
            ],
            "responseElements": {
                "credentials": {
                    "accessKeyId": "REDACTED",
                    "expiration": "Jun 2, 2024, 10:37:12 PM",
                    "sessionToken": "REDACTED",
                },
            },
            "sharedEventID": "95e84e79-100a-40a6-985e-3c9c4b41f622",
            "sourceIPAddress": "lambda.amazonaws.com",
            "userAgent": "lambda.amazonaws.com",
            "userIdentity": {"invokedBy": "lambda.amazonaws.com", "type": "AWSService"},
        },
    ),
]


@panther_managed
class RoleAssumedbyUser(Rule):
    id = "Role.Assumed.by.User-prototype"
    display_name = "SIGNAL - Role Assumed by User"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    tests = role_assumedby_user_tests

    def rule(self, event):
        aws_service = event.deep_get("userIdentity", "type") == "AWSService"
        return all(
            [event.get("eventName") == "AssumeRole", event.deep_get("requestParameters", "roleArn"), not aws_service],
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_cloud_trail_password_policy_discovery_tests: list[RuleTest] = [
    RuleTest(
        name="Non-Discovery Event",
        expected_result=False,
        log={
            "apiversion": "2012-08-10",
            "awsregion": "eu-west-1",
            "eventcategory": "Data",
            "eventid": "5d4b45ed-a15c-41b6-80e9-031729fa060d",
            "eventname": "GetRecords",
            "eventsource": "dynamodb.amazonaws.com",
            "eventtime": "2023-01-10 21:04:02",
            "eventtype": "AwsApiCall",
            "eventversion": "1.08",
            "managementevent": False,
            "useridentity": {"arn": "arn:aws:test_arn"},
        },
    ),
    RuleTest(
        name="Password Discovery ARN",
        expected_result=True,
        log={
            "awsregion": "us-east-1",
            "eventcategory": "Management",
            "eventid": "1808ca3b-4311-4b48-9d1f-21061acb2329",
            "eventname": "GetAccountPasswordPolicy",
            "eventsource": "iam.amazonaws.com",
            "eventtime": "2023-01-10 23:10:06",
            "eventtype": "AwsApiCall",
            "eventversion": "1.08",
            "managementevent": True,
            "useridentity": {"arn": "arn:aws:test_arn"},
        },
    ),
    RuleTest(
        name="Password Discovery Service",
        expected_result=False,
        log={
            "awsregion": "us-east-1",
            "eventType": "AwsServiceEvent",
            "eventcategory": "Management",
            "eventid": "1808ca3b-4311-4b48-9d1f-21061acb2329",
            "eventname": "GetAccountPasswordPolicy",
            "eventsource": "iam.amazonaws.com",
            "eventtime": "2023-01-10 23:10:06",
            "eventversion": "1.08",
            "managementevent": True,
        },
    ),
]


@panther_managed
class AWSCloudTrailPasswordPolicyDiscovery(Rule):
    default_description = "This detection looks for *AccountPasswordPolicy events in AWS CloudTrail logs. If these events occur in a short period of time from the same ARN, it could constitute Password Policy reconnaissance."
    display_name = "AWS CloudTrail Password Policy Discovery"
    reports = {"MITRE ATT&CK": ["TA0007:T1201"]}
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_passwords_account-policy.html"
    default_severity = Severity.INFO
    dedup_period_minutes = 30
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.CloudTrail.Password.Policy.Discovery-prototype"
    threshold = 2
    tests = aws_cloud_trail_password_policy_discovery_tests
    PASSWORD_DISCOVERY_EVENTS = ["GetAccountPasswordPolicy", "UpdateAccountPasswordPolicy", "PutAccountPasswordPolicy"]

    def rule(self, event):
        service_event = event.get("eventType") == "AwsServiceEvent"
        return event.get("eventName") in self.PASSWORD_DISCOVERY_EVENTS and (not service_event)

    def title(self, event):
        user_arn = deep_get(event, "useridentity", "arn", default="<MISSING_ARN>")
        return f"Password Policy Discovery detected in AWS CloudTrail from [{user_arn}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

netskope_admin_user_change_tests: list[RuleTest] = [
    RuleTest(
        name="True positive",
        expected_result=True,
        log={
            "_id": "e5ca619b059fccdd0cfd9398",
            "_insertion_epoch_timestamp": 1702308331,
            "audit_log_event": "Created new admin",
            "count": 1,
            "is_netskope_personnel": True,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {"data_type": "user", "data_values": ["11.22.33.44", "adminsupport@netskope.com"]},
            "timestamp": "2023-12-11 15:25:31.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "adminsupport@netskope.com",
            "user": "adminsupport@netskope.com",
        },
    ),
    RuleTest(
        name="True negative",
        expected_result=False,
        log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Rest API V2 Call",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "incidents",
                "data_values": [200, "POST", "/api/v2/incidents/uba/getuci", "trid=ccb898fgrhvdd0v0lebg"],
            },
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
]


@panther_managed
class NetskopeAdminUserChange(Rule):
    id = "Netskope.AdminUserChange-prototype"
    display_name = "An administrator account was created, deleted, or modified."
    log_types = [LogType.NETSKOPE_AUDIT]
    tags = ["Netskope", "Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0004:T1098"]}
    default_severity = Severity.HIGH
    default_reference = (
        "https://docs.netskope.com/en/netskope-help/admin-console/administration/managing-administrators/"
    )
    default_description = "An administrator account was created, deleted, or modified."
    default_runbook = "An administrator account was created, deleted, or modified.  Validate that this activity is expected and authorized."
    tests = netskope_admin_user_change_tests
    ADMIN_USER_CHANGE_EVENTS = [
        "Created new admin",
        "Added SSO Admin",
        "Edited SSO Admin Record",
        "Created new support admin",
        "Edit admin record",
        "Deleted admin",
        "Enabled admin",
        "Disabled admin",
        "Unlocked admin",
        "Updated admin settings",
        "Deleted Netskope SSO admin",
    ]

    def rule(self, event):
        if event.get("audit_log_event") in self.ADMIN_USER_CHANGE_EVENTS:
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        audit_log_event = event.get("audit_log_event", "<EVENT_NOT_FOUND>")
        return f"User [{user}] performed [{audit_log_event}]"

    def severity(self, event):
        audit_log_event = event.get("audit_log_event", "no_data").lower()
        if "create" in audit_log_event or "add" in audit_log_event or "delete" in audit_log_event:
            return "CRITICAL"
        return "HIGH"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_iam_anything_changed_tests: list[RuleTest] = [
    RuleTest(
        name="IAM Change",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "AttachRolePolicy",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "policyArn": "arn:aws:iam::aws:policy/example-policy",
                "roleName": "LambdaFunctionRole-1111",
            },
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111:example-user",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-user",
                    },
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="IAM Read Only Activity",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "DescribePolicy",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"roleName": "LambdaFunctionRole-1111"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111:example-user",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-user",
                    },
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Error Making IAM Change",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "errorCode": "NoSuchEntity",
            "eventID": "1111",
            "eventName": "AttachRolePolicy",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "policyArn": "arn:aws:iam::aws:policy/example-policy",
                "roleName": "LambdaFunctionRole-1111",
            },
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111:example-user",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-user",
                    },
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailIAMAnythingChanged(Rule):
    id = "AWS.CloudTrail.IAMAnythingChanged-prototype"
    display_name = "IAM Change"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity and Access Management"]
    default_severity = Severity.INFO
    dedup_period_minutes = 720
    default_description = "A change occurred in the IAM configuration. This could be a resource being created, deleted, or modified. This is a high level view of changes, helfpul to indicate how dynamic a certain IAM environment is.\n"
    default_runbook = "Ensure this was an approved IAM configuration change.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/cloudtrail-integration.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_iam_anything_changed_tests
    IAM_CHANGE_ACTIONS = [
        "Add",
        "Attach",
        "Change",
        "Create",
        "Deactivate",
        "Delete",
        "Detach",
        "Enable",
        "Put",
        "Remove",
        "Set",
        "Update",
        "Upload",
    ]

    def rule(self, event):
        # Only check IAM events, as the next check is relatively computationally
        # expensive and can often be skipped
        if not aws_cloudtrail_success(event) or event.get("eventSource") != "iam.amazonaws.com":
            return False
        return any(event.get("eventName", "").startswith(action) for action in self.IAM_CHANGE_ACTIONS)

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

aws_modify_cloud_compute_infrastructure_tests: list[RuleTest] = [
    RuleTest(
        name="Terminate Instance from AssumedRole",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "59e8d6b8-de7b-43ca-961f-0c6f4531fcf0",
            "eventName": "TerminateInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2021-10-29 23:50:09",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["111222333444"],
            "p_any_aws_arns": ["arn:aws:iam::111222333444:role/FakeRole"],
            "p_any_aws_instance_ids": ["i-0d9853f67e40ab80b"],
            "p_any_domain_names": ["ec2.amazonaws.com"],
            "p_event_time": "2021-10-29 23:50:09",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2021-10-29 23:54:06.45",
            "p_row_id": "e6f7bd65083bfeb7feced38f0da18a01",
            "p_source_id": "5f9f0f60-9c56-4027-b93a-8bab3019f0f1",
            "p_source_label": "SomeCloudTrail",
            "readOnly": False,
            "recipientAccountId": "111222333444",
            "requestID": "a520eeaf-c258-4260-954e-b4a976e6c72b",
            "requestParameters": {"instancesSet": {"items": [{"instanceId": "i-0d9853f67e40ab80b"}]}},
            "responseElements": {
                "instancesSet": {
                    "items": [
                        {
                            "currentState": {"code": 32, "name": "shutting-down"},
                            "instanceId": "i-0d9853f67e40ab80b",
                            "previousState": {"code": 16, "name": "running"},
                        },
                    ],
                },
                "requestId": "a520eeaf-c258-4260-954e-b4a976e6c72b",
            },
            "userIdentity": {
                "accountId": "111222333444",
                "arn": "arn:aws:sts::111222333444:assumed-role/SomeRole/AThing",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-29T23:50:08Z", "mfaAuthenticated": "false"},
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Terminate Instance from autoscaling",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "59e8d6b8-de7b-43ca-961f-0c6f4531fcf0",
            "eventName": "TerminateInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2021-10-29 23:50:09",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["111222333444"],
            "p_any_aws_arns": [
                "arn:aws:iam::111222333444:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling",
                "arn:aws:sts::111222333444:assumed-role/AWSServiceRoleForAutoScaling/AutoScaling",
            ],
            "p_any_aws_instance_ids": ["i-0d9853f67e40ab80b"],
            "p_any_domain_names": ["autoscaling.amazonaws.com"],
            "p_event_time": "2021-10-29 23:50:09",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2021-10-29 23:54:06.45",
            "p_row_id": "e6f7bd65083bfeb7feced38f0da18a01",
            "p_source_id": "5f9f0f60-9c56-4027-b93a-8bab3019f0f1",
            "p_source_label": "SomeCloudTrail",
            "readOnly": False,
            "recipientAccountId": "111222333444",
            "requestID": "a520eeaf-c258-4260-954e-b4a976e6c72b",
            "requestParameters": {"instancesSet": {"items": [{"instanceId": "i-0d9853f67e40ab80b"}]}},
            "responseElements": {
                "instancesSet": {
                    "items": [
                        {
                            "currentState": {"code": 32, "name": "shutting-down"},
                            "instanceId": "i-0d9853f67e40ab80b",
                            "previousState": {"code": 16, "name": "running"},
                        },
                    ],
                },
                "requestId": "a520eeaf-c258-4260-954e-b4a976e6c72b",
            },
            "sourceIPAddress": "autoscaling.amazonaws.com",
            "userAgent": "autoscaling.amazonaws.com",
            "userIdentity": {
                "accountId": "111222333444",
                "arn": "arn:aws:sts::111222333444:assumed-role/AWSServiceRoleForAutoScaling/AutoScaling",
                "invokedBy": "autoscaling.amazonaws.com",
                "principalId": "AROATSZWD7TDLUEWEUXXI:AutoScaling",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-29T23:50:08Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "111222333444",
                        "arn": "arn:aws:iam::111222333444:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling",
                        "principalId": "AROATSZWD7TDLUEWEUXXI",
                        "type": "Role",
                        "userName": "AWSServiceRoleForAutoScaling",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Get Partition",
        expected_result=False,
        log={
            "additionalEventData": {
                "insufficientLakeFormationPermissions": ["panther_rule_errors:gsuite_activityevent"],
                "lakeFormationPrincipal": "arn:aws:iam::111222333444:role/panther-Panther-4JL51Q6AU6SH-LogAnal-CompactorRole-W1WCIV3PHU0S",
            },
            "awsRegion": "us-east-1",
            "errorCode": "EntityNotFoundException",
            "errorMessage": "Cannot find partition.",
            "eventID": "8780fc6b-7742-4a45-b757-c351a54c79b8",
            "eventName": "GetPartition",
            "eventSource": "glue.amazonaws.com",
            "eventTime": "2021-10-14 14:21:22",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["111222333444"],
            "p_any_aws_arns": [
                "arn:aws:iam::111222333444:role/panther-Panther-4JL51Q6AU6SH-LogAnal-CompactorRole-W1WCIV3PHU0S",
                "arn:aws:sts::111222333444:assumed-role/panther-Panther-4JL51Q6AU6SH-LogAnal-CompactorRole-W1WCIV3PHU0S/panther-datacatalog-compactor",
            ],
            "p_any_ip_addresses": ["54.90.94.136"],
            "p_event_time": "2021-10-14 14:21:22",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2021-10-14 14:27:06.54",
            "p_row_id": "7235e51c49e780a5a4e281e90c850c",
            "p_source_id": "5f9f0f60-9c56-4027-b93a-8bab3019f0f1",
            "p_source_label": "SomeCloudTrail",
            "readOnly": True,
            "recipientAccountId": "111222333444",
            "requestID": "cdb450f7-1cd8-463b-8449-71274d95a5a3",
            "requestParameters": {
                "databaseName": "panther_rule_errors",
                "partitionValues": ["2021", "10", "13", "19", "1634151600"],
                "tableName": "gsuite_activityevent",
            },
            "sourceIPAddress": "54.90.94.136",
            "userAgent": "aws-sdk-go/1.40.21 (go1.17; linux; amd64) exec-env/AWS_Lambda_go1.x",
            "userIdentity": {
                "accessKeyId": "ASIAJMVY5WC5K4TDFNFA",
                "accountId": "111222333444",
                "arn": "arn:aws:sts::111222333444:assumed-role/panther-Panther-4JL51Q6AU6SH-LogAnal-CompactorRole-W1WCIV3PHU0S/panther-datacatalog-compactor",
                "principalId": "AROA4UN2W2PXWZMJ2L3PC:panther-datacatalog-compactor",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-14T14:20:28Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "111222333444",
                        "arn": "arn:aws:iam::111222333444:role/panther-Panther-4JL51Q6AU6SH-LogAnal-CompactorRole-W1WCIV3PHU0S",
                        "principalId": "AROA4UN2W2PXWZMJ2L3PC",
                        "type": "Role",
                        "userName": "panther-Panther-4JL51Q6AU6SH-LogAnal-CompactorRole-W1WCIV3PHU0S",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Terminate instance From WebUI with assumedRole",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "01f39d3b-4a26-4045-bb36-1e57b7d07997",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-10-14 00:35:36",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123412341234"],
            "p_any_aws_arns": [
                "arn:aws:iam::123412341234:role/SomeRole",
                "arn:aws:sts::123412341234:assumed-role/SomeRole/person",
            ],
            "p_any_aws_instance_ids": ["i-0690cd354a0c3850c"],
            "p_any_aws_tags": ["Name:fake thing whatever"],
            "p_any_domain_names": ["AWS Internal", "ip-10-1-0-14.us-west-2.compute.internal"],
            "p_any_ip_addresses": ["10.1.0.14"],
            "p_any_trace_ids": ["ASIARLIVEKVNGOY5UABO"],
            "p_any_usernames": ["SomeRole"],
            "p_event_time": "2022-10-14 00:35:36",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-10-14 00:38:26.875",
            "p_row_id": "76663c86299fc3f2fa94acfd13f29311",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "SomeCloudTrail",
            "readOnly": False,
            "recipientAccountId": "123412341234",
            "requestID": "557e2d68-904a-4fe7-81d4-33c056444a13",
            "requestParameters": {
                "blockDeviceMapping": {},
                "disableApiStop": False,
                "disableApiTermination": False,
                "ebsOptimized": False,
                "instanceType": "t2.micro",
                "instancesSet": {"items": [{"imageId": "ami-08e2d37b6a0129927", "maxCount": 1, "minCount": 1}]},
                "monitoring": {"enabled": False},
                "networkInterfaceSet": {
                    "items": [
                        {
                            "associatePublicIpAddress": False,
                            "deviceIndex": 0,
                            "groupSet": {"items": [{"groupId": "sg-0077c778d7ad1f5f2"}]},
                            "subnetId": "subnet-0e3a508e43776c435",
                        },
                    ],
                },
                "privateDnsNameOptions": {
                    "enableResourceNameDnsAAAARecord": False,
                    "enableResourceNameDnsARecord": True,
                    "hostnameType": "ip-name",
                },
                "tagSpecificationSet": {
                    "items": [{"resourceType": "instance", "tags": [{"key": "Name", "value": "fake thing whatever"}]}],
                },
            },
            "responseElements": {
                "groupSet": {},
                "instancesSet": {
                    "items": [
                        {
                            "amiLaunchIndex": 0,
                            "architecture": "x86_64",
                            "blockDeviceMapping": {},
                            "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                            "cpuOptions": {"coreCount": 1, "threadsPerCore": 1},
                            "ebsOptimized": False,
                            "enaSupport": True,
                            "enclaveOptions": {"enabled": False},
                            "groupSet": {
                                "items": [{"groupId": "sg-0077c778d7ad1f5f2", "groupName": "launch-wizard-1"}],
                            },
                            "hypervisor": "xen",
                            "imageId": "ami-08e2d37b6a0129927",
                            "instanceId": "i-0690cd354a0c3850c",
                            "instanceState": {"code": 0, "name": "pending"},
                            "instanceType": "t2.micro",
                            "launchTime": 1665707736000,
                            "maintenanceOptions": {"autoRecovery": "default"},
                            "metadataOptions": {
                                "httpEndpoint": "enabled",
                                "httpProtocolIpv4": "enabled",
                                "httpProtocolIpv6": "disabled",
                                "httpPutResponseHopLimit": 1,
                                "httpTokens": "optional",
                                "instanceMetadataTags": "disabled",
                                "state": "pending",
                            },
                            "monitoring": {"state": "disabled"},
                            "networkInterfaceSet": {
                                "items": [
                                    {
                                        "attachment": {
                                            "attachTime": 1665707736000,
                                            "attachmentId": "eni-attach-0b2f21743e26b7c8a",
                                            "deleteOnTermination": True,
                                            "deviceIndex": 0,
                                            "networkCardIndex": 0,
                                            "status": "attaching",
                                        },
                                        "groupSet": {
                                            "items": [
                                                {"groupId": "sg-0077c778d7ad1f5f2", "groupName": "launch-wizard-1"},
                                            ],
                                        },
                                        "interfaceType": "interface",
                                        "ipv6AddressesSet": {},
                                        "macAddress": "02:50:9d:da:61:79",
                                        "networkInterfaceId": "eni-0f4e4d2d71627dd38",
                                        "ownerId": "123412341234",
                                        "privateDnsName": "ip-10-1-0-14.us-west-2.compute.internal",
                                        "privateIpAddress": "10.1.0.14",
                                        "privateIpAddressesSet": {
                                            "item": [
                                                {
                                                    "primary": True,
                                                    "privateDnsName": "ip-10-1-0-14.us-west-2.compute.internal",
                                                    "privateIpAddress": "10.1.0.14",
                                                },
                                            ],
                                        },
                                        "sourceDestCheck": True,
                                        "status": "in-use",
                                        "subnetId": "subnet-0e3a508e43776c435",
                                        "tagSet": {},
                                        "vpcId": "vpc-04fe895571b367c47",
                                    },
                                ],
                            },
                            "placement": {"availabilityZone": "us-west-2a", "tenancy": "default"},
                            "privateDnsName": "ip-10-1-0-14.us-west-2.compute.internal",
                            "privateDnsNameOptions": {
                                "enableResourceNameDnsAAAARecord": False,
                                "enableResourceNameDnsARecord": True,
                                "hostnameType": "ip-name",
                            },
                            "privateIpAddress": "10.1.0.14",
                            "productCodes": {},
                            "rootDeviceName": "/dev/xvda",
                            "rootDeviceType": "ebs",
                            "sourceDestCheck": True,
                            "stateReason": {"code": "pending", "message": "pending"},
                            "subnetId": "subnet-0e3a508e43776c435",
                            "tagSet": {"items": [{"key": "Name", "value": "fake thing whatever"}]},
                            "virtualizationType": "hvm",
                            "vpcId": "vpc-04fe895571b367c47",
                        },
                    ],
                },
                "ownerId": "123412341234",
                "requestId": "557e2d68-904a-4fe7-81d4-33c056444a13",
                "reservationId": "r-0de81af4b677c0252",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIARLIVEKVNGOY5UABO",
                "accountId": "123412341234",
                "arn": "arn:aws:sts::123412341234:assumed-role/SomeRole/person",
                "principalId": "AROARLIVEKVNIRVGDLJWJ:person",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-10-14T00:34:59Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123412341234",
                        "arn": "arn:aws:iam::123412341234:role/SomeRole",
                        "principalId": "AROARLIVEKVNIRVGDLJWJ",
                        "type": "Role",
                        "userName": "SomeRole",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Weird AWS Internal Message",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "Client.DryRunOperation",
            "errorMessage": "Request would have succeeded, but DryRun flag is set.",
            "eventCategory": "Management",
            "eventID": "b32f82e6-7375-4487-85f3-442face5eab4",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-10-13 16:34:53",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123412341234"],
            "p_any_aws_arns": [
                "arn:aws:iam::123412341234:role/funky-role-doing-dryrun",
                "arn:aws:sts::123412341234:assumed-role/funky-role-doing-dryrun/1665678811",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_usernames": ["funky-role-doing-dryrun"],
            "p_event_time": "2022-10-13 16:34:53",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-10-13 16:35:54.489",
            "p_row_id": "6650c5fe395984e0fdb7c1fc138ef507",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "SomeCloudTrail",
            "readOnly": False,
            "recipientAccountId": "123412341234",
            "requestID": "99288e91-d3a5-494b-9fe6-9fdc02646a16",
            "requestParameters": {
                "blockDeviceMapping": {},
                "clientToken": "80a552a2-30ee-419e-bd73-3ee36b1242d6",
                "disableApiStop": False,
                "disableApiTermination": False,
                "instanceType": "m5.xlarge",
                "instancesSet": {"items": [{"imageId": "ami-05074c40f29040248", "maxCount": 1, "minCount": 1}]},
                "launchTemplate": {"launchTemplateId": "lt-064c1a4dbc97b01fc", "version": "5"},
                "monitoring": {"enabled": False},
                "subnetId": "subnet-00559b970d3a60983",
            },
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accountId": "123412341234",
                "arn": "arn:aws:sts::123412341234:assumed-role/funky-role-doing-dryrun/astronomer-managed-1665678811",
                "invokedBy": "AWS Internal",
                "principalId": "AROAZBD2CNPWEWKWTLX67:astronomer-managed-1665678811",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-10-13T16:33:31Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123412341234",
                        "arn": "arn:aws:iam::123412341234:role/funky-role-doing-dryrun",
                        "principalId": "AROAZBD2CNPWEWKWTLX67",
                        "type": "Role",
                        "userName": "funky-role-doing-dryrun",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSModifyCloudComputeInfrastructure(Rule):
    default_description = (
        "Detection when EC2 compute infrastructure is modified outside of expected automation methods."
    )
    display_name = "AWS Modify Cloud Compute Infrastructure"
    enabled = False
    default_reference = "https://attack.mitre.org/techniques/T1578/"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1578"]}
    tags = ["Configuration Required"]
    default_runbook = "This detection reports on eventSource ec2 Change events. This detection excludes Cross-Service change events.  As such, this detection will perform well in environments where changes are expected to originate only from AWS service entities.\nThis detection will emit alerts frequently in environments where users are making ec2 related changes.\n"
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Modify.Cloud.Compute.Infrastructure-prototype"
    tests = aws_modify_cloud_compute_infrastructure_tests
    EC2_CRUD_ACTIONS = {
        "AssociateIamInstanceProfile",
        "AssociateInstanceEventWindow",
        "BundleInstance",
        "CancelSpotInstanceRequests",
        "ConfirmProductInstance",
        "CreateInstanceEventWindow",
        "CreateInstanceExportTask",
        "DeleteInstanceEventWindow",
        "DeregisterInstanceEventNotificationAttributes",
        "DisassociateIamInstanceProfile",
        "DisassociateInstanceEventWindow",
        "ImportInstance",
        "ModifyInstanceAttribute",
        "ModifyInstanceCapacityReservationAttributes",
        "ModifyInstanceCreditSpecification",
        "ModifyInstanceEventStartTime",
        "ModifyInstanceEventWindow",
        "ModifyInstanceMaintenanceOptions",
        "ModifyInstanceMetadataOptions",
        "ModifyInstancePlacement",
        "MonitorInstances",
        "RegisterInstanceEventNotificationAttributes",
        "ReportInstanceStatus",
        "RequestSpotInstances",
        "ResetInstanceAttribute",
        "RunInstances",
        "RunScheduledInstances",
        "StartInstances",
        "StopInstances",
        "TerminateInstances",
        "UnmonitorInstances",
    }

    def rule(self, event):
        # Disqualify any eventSource that is not ec2
        if event.get("eventSource", "") != "ec2.amazonaws.com":
            return False
        # Disqualify AWS Service-Service operations, which can appear in a variety of forms
        if (
            event.get("sourceIPAddress", "").endswith(".amazonaws.com")
            or deep_get(event, "userIdentity", "type", default="") == "AWSService"
            or deep_get(event, "userIdentity", "invokedBy", default="") == "AWS Internal"
            or deep_get(event, "userIdentity", "invokedBy", default="").endswith(".amazonaws.com")
        ):
            # FYI there is a weird quirk in the sourceIPAddress field of CloudTrail
            #  events with ec2.amazonaws.com as the source name where users of the
            #  web-console will have their sourceIPAddress recorded as "AWS Internal"
            #  though their userIdentity will be more normal.
            #  Example cloudtrail event in the "Terminate instance From WebUI with assumedRole" test
            return False
        # Dry run operations get logged as SES Internal in the sourceIPAddress
        #  but not in the invokedBy field
        if event.get("errorCode", "") == "Client.DryRunOperation":
            return False
        # Disqualify any eventNames that do not Include instance
        # and events that have readOnly set to false
        if event.get("eventName", "") in self.EC2_CRUD_ACTIONS:
            return True
        return False

    def title(self, event):
        items = deep_get(event, "requestParameters", "instancesSet", "items", default=[{"instanceId": "none"}])
        return f"AWS Event [{event.get('eventName')}] Instance ID [{items[0].get('instanceId')}] AWS Account ID [{event.get('recipientAccountId')}]"

    def alert_context(self, event):
        items = deep_get(event, "requestParameters", "instancesSet", "items", default=[{"instanceId": "none"}])
        return {
            "awsRegion": event.get("awsRegion"),
            "eventName": event.get("eventName"),
            "recipientAccountId": event.get("recipientAccountId"),
            "instanceId": items[0].get("instanceId"),
        }



from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import okta_alert_context

okta_rate_limits_tests: list[RuleTest] = [
    RuleTest(
        name="system.org.ratelimit.warning",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc456",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
            "client": {
                "device": "Unknown",
                "ipAddress": "1.2.3.4",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "rateLimitScopeType": "ORG",
                    "rateLimitSecondsToReset": "11",
                    "requestId": "abc1234",
                    "requestUri": "/api/v1/users",
                    "threshold": "500",
                    "timeSpan": "1",
                    "timeUnit": "MINUTES",
                    "url": "/api/v1/users?limit=100",
                    "userId": "abc1234",
                    "warningPercent": "80",
                },
            },
            "displaymessage": "Rate limit warning",
            "eventtype": "system.org.rate_limit.warning",
            "legacyeventtype": "core.framework.ratelimit.warning",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-08-26 07:51:24.601",
            "request": {"ipChain": [{"ip": "1.2.3.4", "version": "V4"}]},
            "securitycontext": {},
            "severity": "INFO",
            "target": [{"id": "/api/v1/users", "type": "URL Pattern"}, {"id": "abc12345", "type": "Bucket Uuid"}],
            "transaction": {"detail": {"requestApiTokenId": "A.12345.DEFGH"}, "id": "ABC1235", "type": "WEB"},
            "uuid": "abc-1234-abcd",
            "version": "0",
        },
    ),
    RuleTest(
        name="system.operation.ratelimit.violation",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc456",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
            "client": {
                "device": "Unknown",
                "ipAddress": "1.2.3.4",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "ABCDFDE",
                    "dtHash": "adfalsjflasfjsdfd",
                    "operationRateLimitScopeType": "User",
                    "operationRateLimitSecondsToReset": "10",
                    "operationRateLimitSubtype": "Authenticated user",
                    "operationRateLimitThreshold": "40",
                    "operationRateLimitTimeSpan": "10",
                    "operationRateLimitTimeUnit": "SECONDS",
                    "operationRateLimitType": "Web request",
                    "requestId": "asfsagadffdaf",
                    "requestUri": "/app/google/",
                    "url": "/app/google/",
                },
            },
            "displaymessage": "Operation rate limit violation",
            "eventtype": "system.operation.rate_limit.violation",
            "outcome": {"reason": "Too many requests attempted by an individual user", "result": "DENY"},
            "published": "2022-08-29 16:07:26.592",
            "request": {"ipChain": [{"ip": "1.2.3.4", "version": "V4"}]},
            "securitycontext": {},
            "severity": "WARN",
            "target": [{"id": "/app/{app}/{key}/", "type": "URL Pattern"}],
            "transaction": {"detail": {}, "id": "YABCDE", "type": "WEB"},
            "uuid": "asdfdashh",
            "version": "0",
        },
    ),
    RuleTest(
        name="application.integration.rate_limit_exceeded",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc456",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
            "client": {
                "device": "Unknown",
                "ipAddress": "1.2.3.4",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                "zone": "null",
            },
            "debugcontext": {"debugData": {}},
            "eventtype": "application.integration.rate_limit_exceeded",
            "legacyeventtype": "app.api.error.rate.limit.exceeded",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-10 17:19:58.423",
            "request": {},
            "securitycontext": {},
            "severity": "INFO",
            "target": [{"alternateId": "App ", "displayName": "App", "id": "12345", "type": "AppInstance"}],
            "transaction": {"detail": {}, "id": "sdfg", "type": "JOB"},
            "uuid": "aaa-bb-ccc",
            "version": "0",
        },
    ),
    RuleTest(
        name="Non event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpsons",
                "id": "00ABC123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 11.111, "lon": -70},
                    "postalCode": "1234",
                    "state": "California",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "ABC123",
                    "deviceFingerprint": "009988771ABC",
                    "dtHash": "123abc1234",
                    "requestId": "abc-111-adf",
                    "requestUri": "/idp/idx/identify",
                    "threatSuspected": "false",
                    "url": "/idp/idx/identify?",
                },
            },
            "displaymessage": "Group Privilege granted",
            "eventtype": "group.privilege.grant",
            "legacyeventtype": "group.privilege.grant",
            "outcome": {"result": "FAILURE"},
            "published": "2022-12-13 00:58:19.811",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 11.111, "lon": -70},
                            "postalCode": "1234",
                            "state": "California",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 11351,
                "asOrg": "charter communications inc",
                "domain": "rr.com",
                "isProxy": False,
                "isp": "charter communications inc",
            },
            "severity": "WARN",
            "target": [{"alternateId": "App (123)", "displayName": "App (123)", "id": "12345", "type": "AppInstance"}],
            "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
            "uuid": "aa-11-22-33-44-bb",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaRateLimits(Rule):
    default_description = "Potential DoS/Bruteforce attack or hitting limits (system degradation)"
    display_name = "Okta Rate Limits"
    default_severity = Severity.HIGH
    tags = ["Credential Access", "Brute Force", "Impact", "Network Denial of Service"]
    reports = {"MITRE ATT&CK": ["TA0006:T1110", "TA0040:T1498"]}
    default_reference = "https://developer.okta.com/docs/reference/rl-system-log-events/"
    dedup_period_minutes = 360
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.Rate.Limits-prototype"
    tests = okta_rate_limits_tests
    DETECTION_EVENTS = [
        "app.oauth2.client_id_rate_limit_warning",
        "application.integration.rate_limit_exceeded",
        "system.client.rate_limit.*",
        "system.client.concurrency_rate_limit.*",
        "system.operation.rate_limit.*",
        "system.org.rate_limit.*",
        "core.concurrency.org.limit.violation",
    ]

    def rule(self, event):
        eventtype = event.get("eventtype", "")
        for detection_event in self.DETECTION_EVENTS:
            if fnmatch(eventtype, detection_event):
                return True
        return False

    def title(self, event):
        return f"Okta Rate Limit Event: [{event.get('eventtype', '')}] by [{event.get('actor', {}).get('alternateId', '<id-not-found>')}]"

    def severity(self, event):
        if event.get("severity", "") == "INFO":
            return "INFO"
        eventtype = event.get("eventtype", "")
        if "notification" in eventtype:
            return "LOW"
        if "warning" in eventtype:
            return "MEDIUM"
        if "violation" in eventtype:
            return "HIGH"
        return "DEFAULT"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_unsupported_mac_os_tests: list[RuleTest] = [
    RuleTest(
        name="MacOS out of date",
        expected_result=True,
        log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.14.2",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "0",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "platform": "darwin",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "Not Supported",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_vuln-management_os_version",
            "unixTime": "1536682461",
        },
    ),
    RuleTest(
        name="MacOS up to date",
        expected_result=False,
        log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.15.1",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "2",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "platform": "darwin",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "10.15.2",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_vuln-management_os_version",
            "unixTime": "1536682461",
        },
    ),
]


@panther_managed
class OsqueryUnsupportedMacOS(Rule):
    id = "Osquery.UnsupportedMacOS-prototype"
    display_name = "Unsupported macOS version"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "Compliance"]
    default_severity = Severity.LOW
    default_description = (
        "Check that all laptops on the corporate environment are on a version of MacOS supported by IT.\n"
    )
    default_runbook = "Update the MacOs version"
    default_reference = "https://support.apple.com/en-eg/HT201260"
    summary_attributes = ["name", "hostIdentifier", "action"]
    tests = osquery_unsupported_mac_os_tests
    SUPPORTED_VERSIONS = ["10.15.1", "10.15.2", "10.15.3"]

    def rule(self, event):
        return (
            event.get("name") == "pack_vuln-management_os_version"
            and deep_get(event, "columns", "platform") == "darwin"
            and (deep_get(event, "columns", "version") not in self.SUPPORTED_VERSIONS)
            and (event.get("action") == "added")
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_sharing_settings_updated_tests: list[RuleTest] = [
    RuleTest(
        name="Sharing Enabled",
        expected_result=True,
        log={
            "event": {
                "actor": {
                    "id": "c16137bb-5078-4eac-b026-5cbd2f9a027a",
                    "object": "user",
                    "person": {"email": "aaron@example.com"},
                    "type": "person",
                },
                "details": {"state": "enabled"},
                "id": "91b29a4b-4978-40e1-ab56-40221f801ce5",
                "ip_address": "11.22.33.44",
                "platform": "web",
                "timestamp": "2023-12-13 16:39:06.860000000",
                "type": "workspace.settings.allow_guests_setting_updated",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            },
        },
    ),
    RuleTest(
        name="Sharing Disabled",
        expected_result=False,
        log={
            "event": {
                "actor": {
                    "id": "c16137bb-5078-4eac-b026-5cbd2f9a027a",
                    "object": "user",
                    "person": {"email": "aaron@example.com"},
                    "type": "person",
                },
                "details": {
                    "state": "disabled",
                    "target": {"id": "a70a4074-5cac-4fc5-8e59-109df81e5a93", "name": "R&D", "object": "teamspace"},
                },
                "id": "91b29a4b-4978-40e1-ab56-40221f801ce5",
                "ip_address": "11.22.33.44",
                "platform": "web",
                "timestamp": "2023-12-13 16:39:06.860000000",
                "type": "teamspace.settings.allow_guests_setting_updated",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            },
        },
    ),
]


@panther_managed
class NotionSharingSettingsUpdated(Rule):
    id = "Notion.SharingSettingsUpdated-prototype"
    display_name = "Notion Sharing Settings Updated"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Exfiltration"]
    default_description = "A Notion User enabled sharing for a Workspace or Teamspace."
    default_severity = Severity.MEDIUM
    default_runbook = "Possible Data Exfiltration. Follow up with the Notion User to determine if this was done for a valid business reason."
    tests = notion_sharing_settings_updated_tests
    EVENTS = (
        "teamspace.settings.allow_public_page_sharing_setting_updated",
        "teamspace.settings.allow_guests_setting_updated",
        "teamspace.settings.allow_content_export_setting_updated",
        "workspace.settings.allow_public_page_sharing_setting_updated",
        "workspace.settings.allow_guests_setting_updated",
        "workspace.settings.allow_content_export_setting_updated",
    )

    def rule(self, event):
        return all(
            [
                event.deep_get("event", "type", default="") in self.EVENTS,
                event.deep_get("event", "details", "state", default="") == "enabled",
            ],
        )

    def title(self, event):
        actor = event.deep_get("event", "actor", "person", "email", default="NO_ACTOR_FOUND")
        action = event.deep_get("event", "type", default="NO.EVENT.FOUND").split(".")[2]
        teamspace = event.deep_get("event", "details", "target", "name", default=None)
        if teamspace:
            return f"[{actor}] enabled [{action}] for [{teamspace}] Teamspace"
        return f"[{actor}] enabled [{action}] for Workspace"

    def alert_context(self, event):
        return notion_alert_context(event)



from datetime import timedelta

from panther_detection_helpers.caching import get_counter, increment_counter, reset_counter

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_high_risk_login_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "event_type_id": "6",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginHighRiskLogin(Rule):
    id = "OneLogin.HighRiskLogin-prototype"
    display_name = "OneLogin High Risk Login"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin"]
    default_severity = Severity.MEDIUM
    default_description = "A OneLogin user successfully logged in after a failed high-risk login attempt."
    default_reference = "https://resources.onelogin.com/OneLogin_RiskBasedAuthentication-WP-v5.pdf"
    default_runbook = "Investigate whether this was caused by expected user activity."
    summary_attributes = ["account_id", "event_type_id", "user_name", "user_id"]
    tests = one_login_high_risk_login_tests
    THRESH_TTL = timedelta(minutes=10).total_seconds()

    def rule(self, event):
        # Filter events down to successful and failed login events
        if not event.get("user_id") or str(event.get("event_type_id")) not in ["5", "6"]:
            return False
        event_key = self.get_key(event)
        # check risk associated with this event
        if event.get("risk_score", 0) > 50:
            # a failed authentication attempt with high risk score
            if str(event.get("event_type_id")) == "6":
                # update a counter for this user's failed login attempts with a high risk score
                increment_counter(event_key, event.event_time_epoch() + self.THRESH_TTL)
        # Trigger alert if this user recently
        # failed a high risk login
        if str(event.get("event_type_id")) == "5":
            if get_counter(event_key) > 0:
                reset_counter(event_key)
                return True
        return False

    def get_key(self, event):
        return __name__ + ":" + event.get("user_name", "<UNKNOWN_USER>")

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] successfully logged in after a failed high risk login event"



from datetime import timedelta

from panther_detection_helpers.caching import add_to_string_set, get_string_set, put_string_set

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import is_ip_in_network

one_login_active_login_activity_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "event_type_id": "6",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="Shared IP Login Event",
        expected_result=False,
        log={
            "event_type_id": "5",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "ipaddr": "192.168.1.1",
        },
    ),
]


@panther_managed
class OneLoginActiveLoginActivity(Rule):
    id = "OneLogin.ActiveLoginActivity-prototype"
    display_name = "OneLogin Active Login Activity"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Lateral Movement:Use Alternate Authentication Material"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    default_description = "Multiple user accounts logged in from the same ip address."
    default_reference = "https://support.onelogin.com/kb/4271392/user-policies"
    default_runbook = "Investigate whether multiple user's logging in from the same ip address is expected. Determine if this ip address should be added to the SHARED_IP_SPACE array."
    summary_attributes = ["account_id", "user_name", "user_id"]
    tests = one_login_active_login_activity_tests
    THRESH = 2
    THRESH_TTL = timedelta(hours=12).total_seconds()
    # Safelist for IP Subnets to ignore in this ruleset
    # Each entry in the list should be in CIDR notation
    # This should include any source ip addresses
    # that are shared among users such as:
    # proxy servers, the public corporate ip space,
    # scanner ips etc
    SHARED_IP_SPACE = ["192.168.0.0/16"]

    def rule(self, event):
        # Pre-filter: event_type_id = 5 is login events.
        if str(event.get("event_type_id")) != "5" or not event.get("ipaddr") or (not event.get("user_id")):
            return False
        # We expect to see multiple user logins from these shared, common ip addresses
        if is_ip_in_network(event.get("ipaddr"), self.SHARED_IP_SPACE):
            return False
        # This tracks multiple successful logins for different accounts from the same ip address
        # First, keep a list of unique user ids that have logged in from this ip address
        event_key = self.get_key(event)
        user_ids = get_string_set(event_key)
        # the user id of the user that has just logged in
        user_id = str(event.get("user_id"))
        if not user_ids:
            # store this as the first user login from this ip address
            put_string_set(event_key, [user_id], epoch_seconds=event.event_time_epoch() + self.THRESH_TTL)
            return False
        # add a new username if this is a unique user from this ip address
        if user_id not in user_ids:
            user_ids = add_to_string_set(event_key, user_id, epoch_seconds=event.event_time_epoch() + self.THRESH_TTL)
        return len(user_ids) > self.THRESH

    def get_key(self, event):
        return __name__ + ":" + event.get("ipaddr", "<UNKNOWN_IP>")

    def title(self, event):
        return f"Unusual logins in OneLogin for multiple users from ip [{event.get('ipaddr', '<UNKNOWN_IP>')}]"



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get

audit_log_type_map = {
    "user_login": event_type.SUCCESSFUL_LOGIN,
    "user_logout": event_type.SUCCESSFUL_LOGOUT,
    "user_created": event_type.USER_ACCOUNT_CREATED,
    "twosv_disabled_for_user": event_type.MFA_DISABLED,
    "group_created": event_type.USER_GROUP_CREATED,
    "group_deleted": event_type.USER_GROUP_DELETED,
    "user_granted_role": event_type.USER_ROLE_MODIFIED,
    "user_revoked_role": event_type.USER_ROLE_DELETED,
}


def get_event_type(event):
    audit_log_type = deep_get(event, "AuditLog", "Type")
    matched = audit_log_type_map.get(audit_log_type)
    if matched is not None:
        return matched

    if audit_log_type in ("added_org_admin", "group_granted_admin_access"):
        return event_type.ADMIN_ROLE_ASSIGNED

    return None


class StandardAtlassianAudit(DataModel):
    id: str = "Standard.Atlassian.Audit"
    display_name: str = "Atlassian Audit Logs"
    enabled: bool = True
    log_types: list[str] = [LogType.ATLASSIAN_AUDIT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.EventActor.Name"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="$.EventLocation.IP"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import lookup_aws_account_name

aws_cloud_trail_codebuild_project_made_public_tests: list[RuleTest] = [
    RuleTest(
        name="CodeBuild Project Made Public",
        expected_result=True,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "111111111111",
                "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                "accountId": "111122223333",
                "accessKeyId": "ASIAXXXXXXXXXXXX",
                "sessionContext": {
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2021-08-18T14:54:53Z",
            "eventSource": "codebuild.amazonaws.com",
            "eventName": "UpdateProjectVisibility",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "1.1.1.1",
            "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
            "requestParameters": {
                "projectVisibility": "PUBLIC_READ",
                "projectArn": "arn:aws:codebuild:us-east-1:111122223333:project/testproject1234",
                "resourceAccessRole": "arn:aws:iam::111122223333:role/service-role/test",
            },
            "responseElements": None,
            "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
            "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "111122223333",
            "eventCategory": "Management",
        },
    ),
    RuleTest(
        name="CodeBuild Project Made Private",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "111111111111",
                "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                "accountId": "111122223333",
                "accessKeyId": "ASIAXXXXXXXXXXXX",
                "sessionContext": {
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2021-08-18T14:54:53Z",
            "eventSource": "codebuild.amazonaws.com",
            "eventName": "UpdateProjectVisibility",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "1.1.1.1",
            "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
            "requestParameters": {
                "projectVisibility": "PRIVATE",
                "projectArn": "arn:aws:codebuild:us-east-1:111122223333:project/testproject1234",
                "resourceAccessRole": "arn:aws:iam::111122223333:role/service-role/test",
            },
            "responseElements": None,
            "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
            "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "111122223333",
            "eventCategory": "Management",
        },
    ),
    RuleTest(
        name="Not a UpdateProjectVisibility event",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "111111111111",
                "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                "accountId": "111122223333",
                "accessKeyId": "ASIAXXXXXXXXXXXX",
                "sessionContext": {
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2021-08-18T14:54:53Z",
            "eventSource": "codebuild.amazonaws.com",
            "eventName": "CreateProject",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "1.1.1.1",
            "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
            "responseElements": None,
            "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
            "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "111122223333",
            "eventCategory": "Management",
        },
    ),
]


@panther_managed
class AWSCloudTrailCodebuildProjectMadePublic(Rule):
    id = "AWS.CloudTrail.CodebuildProjectMadePublic-prototype"
    display_name = "CodeBuild Project made Public"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    tags = ["AWS", "Security Control", "Exfiltration:Exfiltration Over Web Service"]
    default_severity = Severity.HIGH
    default_description = "An AWS CodeBuild Project was made publicly accessible\n"
    default_runbook = "TBD"
    default_reference = "https://docs.aws.amazon.com/codebuild/latest/userguide/public-builds.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_codebuild_project_made_public_tests

    def rule(self, event):
        return (
            event["eventName"] == "UpdateProjectVisibility"
            and deep_get(event, "requestParameters", "projectVisibility") == "PUBLIC_READ"
        )

    def title(self, event):
        return f"AWS CodeBuild Project made Public by {deep_get(event, 'userIdentity', 'arn')} in account {lookup_aws_account_name(deep_get(event, 'recipientAccountId'))}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import crowdstrike_detection_alert_context

crowdstrike_wmi_query_detection_tests: list[RuleTest] = [
    RuleTest(
        name="Benign wmic",
        expected_result=False,
        log={
            "aid": "1234567890abcdefg654321",
            "aip": "11.10.9.8",
            "cid": "abcdefghijklmnop123467890",
            "configbuild": "1007.3.0016606.11",
            "configstatehash": "3799024366",
            "entitlements": "15",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "5540c470218d209b7c3eca3d12e190580814d566",
                "CommandLine": "C:\\Windows\\System32\\wmic.exe nothing",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3799024366",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\wmic.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "0",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682106752.006",
                "ProcessSxsFlags": "64",
                "RawProcessId": "1468",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "6364981533",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                "TargetProcessId": "4390327988",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "1234567890abcdefg654321",
                "aip": "11.10.9.8",
                "cid": "abcdefghijklmnop123467890",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                "name": "ProcessRollup2V19",
                "timestamp": "1682106752722",
            },
            "event_platform": "Win",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
            "name": "ProcessRollup2V19",
            "p_any_ip_addresses": ["11.10.9.8"],
            "p_any_md5_hashes": [
                "5fd22b915c232378e567160d641cc9f2",
                "1234567890abcdefg654321",
                "abcdefghijklmnop123467890",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
            "p_any_trace_ids": ["4295752857", "1234567890abcdefg654321", "abcdefghijklmnop123467890"],
            "p_event_time": "2023-04-21 19:52:32.722",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-04-21 20:05:52.94",
            "p_row_id": "7ac82dbb43a99bfec196bdda178c8101",
            "p_schema_version": 0,
            "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
            "p_source_label": "Crowdstrike-FDR-Dev",
            "timestamp": "2023-04-21 19:52:32.722",
            "treeid": "4295752857",
        },
    ),
    RuleTest(
        name="Other",
        expected_result=False,
        log={
            "aid": "1234567890abcdefg654321",
            "aip": "11.10.9.8",
            "cid": "abcdefghijklmnop123467890",
            "configbuild": "1007.3.0016606.11",
            "configstatehash": "3799024366",
            "entitlements": "15",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "5540c470218d209b7c3eca3d12e190580814d566",
                "CommandLine": "C:\\Windows\\System32\\nc.exe -n 1.1.1.1 80",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3799024366",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\nc.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "0",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682106752.006",
                "ProcessSxsFlags": "64",
                "RawProcessId": "1468",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "6364981533",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                "TargetProcessId": "4390327988",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "1234567890abcdefg654321",
                "aip": "11.10.9.8",
                "cid": "abcdefghijklmnop123467890",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                "name": "ProcessRollup2V19",
                "timestamp": "1682106752722",
            },
            "event_platform": "Win",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
            "name": "ProcessRollup2V19",
            "p_any_ip_addresses": ["11.10.9.8"],
            "p_any_md5_hashes": [
                "5fd22b915c232378e567160d641cc9f2",
                "1234567890abcdefg654321",
                "abcdefghijklmnop123467890",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
            "p_any_trace_ids": ["4295752857", "1234567890abcdefg654321", "abcdefghijklmnop123467890"],
            "p_event_time": "2023-04-21 19:52:32.722",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-04-21 20:05:52.94",
            "p_row_id": "7ac82dbb43a99bfec196bdda178c8101",
            "p_schema_version": 0,
            "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
            "p_source_label": "Crowdstrike-FDR-Dev",
            "timestamp": "2023-04-21 19:52:32.722",
            "treeid": "4295752857",
        },
    ),
    RuleTest(
        name="wmic get name password",
        expected_result=True,
        log={
            "aid": "1234567890abcdefg654321",
            "aip": "11.10.9.8",
            "cid": "abcdefghijklmnop123467890",
            "configbuild": "1007.3.0016606.11",
            "configstatehash": "3799024366",
            "entitlements": "15",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "5540c470218d209b7c3eca3d12e190580814d566",
                "CommandLine": "C:\\Windows\\System32\\wmic.exe useraccount get name,password",
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3799024366",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\wmic.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "0",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682106752.006",
                "ProcessSxsFlags": "64",
                "RawProcessId": "1468",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "6364981533",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                "TargetProcessId": "4390327988",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "1234567890abcdefg654321",
                "aip": "11.10.9.8",
                "cid": "abcdefghijklmnop123467890",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                "name": "ProcessRollup2V19",
                "timestamp": "1682106752722",
            },
            "event_platform": "Win",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
            "name": "ProcessRollup2V19",
            "p_any_ip_addresses": ["11.10.9.8"],
            "p_any_md5_hashes": [
                "5fd22b915c232378e567160d641cc9f2",
                "1234567890abcdefg654321",
                "abcdefghijklmnop123467890",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
            "p_any_trace_ids": ["4295752857", "1234567890abcdefg654321", "abcdefghijklmnop123467890"],
            "p_event_time": "2023-04-21 19:52:32.722",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-04-21 20:05:52.94",
            "p_row_id": "7ac82dbb43a99bfec196bdda178c8101",
            "p_schema_version": 0,
            "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
            "p_source_label": "Crowdstrike-FDR-Dev",
            "timestamp": "2023-04-21 19:52:32.722",
            "treeid": "4295752857",
        },
    ),
    RuleTest(
        name="wmic process create",
        expected_result=True,
        log={
            "aid": "1234567890abcdefg654321",
            "aip": "11.10.9.8",
            "cid": "abcdefghijklmnop123467890",
            "configbuild": "1007.3.0016606.11",
            "configstatehash": "3799024366",
            "entitlements": "15",
            "event": {
                "AuthenticationId": "293628",
                "AuthenticodeHashData": "5540c470218d209b7c3eca3d12e190580814d566",
                "CommandLine": 'C:\\Windows\\System32\\wmic.exe process call create "cmd.exe /c malware.exe"',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3799024366",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\wmic.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "0",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682106752.006",
                "ProcessSxsFlags": "64",
                "RawProcessId": "1468",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                "SessionId": "2",
                "SignInfoFlags": "8683538",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "6364981533",
                "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                "TargetProcessId": "4390327988",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "1234567890abcdefg654321",
                "aip": "11.10.9.8",
                "cid": "abcdefghijklmnop123467890",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                "name": "ProcessRollup2V19",
                "timestamp": "1682106752722",
            },
            "event_platform": "Win",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
            "name": "ProcessRollup2V19",
            "p_any_ip_addresses": ["11.10.9.8"],
            "p_any_md5_hashes": [
                "5fd22b915c232378e567160d641cc9f2",
                "1234567890abcdefg654321",
                "abcdefghijklmnop123467890",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
            "p_any_trace_ids": ["4295752857", "1234567890abcdefg654321", "abcdefghijklmnop123467890"],
            "p_event_time": "2023-04-21 19:52:32.722",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-04-21 20:05:52.94",
            "p_row_id": "7ac82dbb43a99bfec196bdda178c8101",
            "p_schema_version": 0,
            "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
            "p_source_label": "Crowdstrike-FDR-Dev",
            "timestamp": "2023-04-21 19:52:32.722",
            "treeid": "4295752857",
        },
    ),
]


@panther_managed
class CrowdstrikeWMIQueryDetection(Rule):
    default_description = "Detects execution of WMI queries involving information gathering or actions on remote systems, which could indicate reconnaissance or lateral movement."
    display_name = "Crowdstrike WMI Query Detection"
    default_runbook = "Investigate the endpoint for signs of WMI query execution. Review the executed query and the associated user account."
    default_reference = "https://learn.microsoft.com/en-us/windows/win32/wmisdk/querying-wmi"
    default_severity = Severity.LOW
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    id = "Crowdstrike.WMI.Query.Detection-prototype"
    tests = crowdstrike_wmi_query_detection_tests
    WMIC_SIGNATURES = ["get", "list", "process call create", "cmd.exe", "powershell.exe", "command.exe"]

    def rule(self, event):
        if event.deep_get("event", "event_simpleName") == "ProcessRollup2":
            if event.deep_get("event", "event_platform") == "Win":
                if event.deep_get("event", "ImageFileName", default="").split("\\")[-1] == "wmic.exe":
                    command_line = event.deep_get("event", "CommandLine", default="")
                    for signature in self.WMIC_SIGNATURES:
                        if signature in command_line:
                            return True
        return False

    def title(self, event):
        cmd = event.deep_get("event", "CommandLine", default="<COMMAND_LINE_NOT_FOUND>")
        aid = event.get("aid", "<AID_NOT_FOUND>")
        return f"Crowdstrike: WMIC Query [{cmd}] performed on aid [{aid}]"

    def alert_context(self, event):
        return crowdstrike_detection_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get, pattern_match_list
from pypanther.helpers.panther_default import aws_cloudtrail_success

awsec2_manual_security_group_change_tests: list[RuleTest] = [
    RuleTest(
        name="AWS Console - Ingress SG Authorization",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "504b492f-7832-406b-a4fd-45a13e48adc4",
            "eventName": "AuthorizeSecurityGroupIngress",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2021-01-24 04:55:45.000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "112233445566",
            "requestID": "91f34d65-513d-4e9f-a3de-e8d27f7ee4b2",
            "requestParameters": {
                "groupId": "sg-04f0b44316f7d2471",
                "ipPermissions": {
                    "items": [
                        {
                            "ipRanges": {"items": [{"cidrIp": "0.0.0.0/16"}]},
                            "prefixListIds": {},
                            "fromPort": "443",
                            "toPort": "443",
                            "groups": {},
                            "ipProtocol": "tcp",
                            "ipv6Ranges": {},
                        },
                    ],
                },
            },
            "responseElements": {"_return": True, "requestId": "91f34d65-513d-4e9f-a3de-e8d27f7ee4b2"},
            "sourceIPAddress": "136.25.37.134",
            "userAgent": "console.ec2.amazonaws.com",
            "userIdentity": {
                "type": "AssumedRole",
                "principalid": "ARORJ4ULULLE0EEJAAKDO:alan",
                "arn": "arn:aws:sts::112233445566:assumed-role/TestAdmin/alan",
                "accountid": "112233445566",
                "accesskeyid": "ASIASWJRT64Z7ZLFLJNI",
                "sessioncontext": {
                    "attributes": {"mfaauthenticated": "true", "creationdate": "2021-01-24T04:55:10Z"},
                    "sessionissuer": {
                        "type": "Role",
                        "principalid": "ARORJ4ULULLE0EEJAAKDO",
                        "arn": "arn:aws:iam::112233445566:role/TestAdmin",
                        "accountid": "112233445566",
                        "username": "TestAdmin",
                    },
                },
            },
            "p_event_time": "2021-01-24 04:55:45.000",
            "p_parse_time": "2021-01-24 05:02:58.358",
            "p_log_type": "AWS.CloudTrail",
            "p_row_id": "1a57ff7ade26aaf5a1a4d7d20775",
            "p_source_id": "e55677c6-7ef5-4541-a443-0d0f17eec19f",
            "p_source_label": "CloudTrail Test",
            "p_any_ip_addresses": ["136.25.37.134"],
            "p_any_aws_account_ids": ["112233445566"],
            "p_any_aws_arns": [
                "arn:aws:iam::112233445566:role/TestAdmin",
                " arn:aws:sts::112233445566:assumed-role/TestAdmin/alan",
            ],
        },
    ),
    RuleTest(
        name="Terraform Security Group Creation",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROAIZCYMDBM4SJU6XXXX:ryan@example.com",
                "arn": "arn:aws:sts::112233445566:assumed-role/Operator/ryan@example.com",
                "accountId": "112233445566",
                "accessKeyId": "ASIAWDWBPTM3RJH7XXXX",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROAIZCYMDBM4SJU65M54",
                        "arn": "arn:aws:iam::112233445566:role/Operator",
                        "accountId": "112233445566",
                        "userName": "Operator",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2020-04-30T23:50:12Z"},
                },
            },
            "eventTime": "2020-04-30T23:51:06Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "CreateSecurityGroup",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "100.1.114.142",
            "userAgent": "aws-sdk-go/1.29.7 (go1.13.7; darwin; amd64) APN/1.0 HashiCorp/1.0 Terraform/0.12.24 (+https://www.terraform.io)",
            "requestParameters": {
                "groupName": "prod-webapp-webserver-security",
                "groupDescription": "prod-webapp-webserver Security SG",
                "vpcId": "vpc-a1a044c7",
            },
            "responseElements": {
                "requestId": "594fe3e3-0f74-4085-9336-189b36a1cd8c",
                "_return": True,
                "groupId": "sg-04d018d184a18f647",
            },
            "requestID": "594fe3e3-0f74-4085-9336-189b36a1cd8c",
            "eventID": "52601748-2659-4ed8-b5fd-c530547b07ec",
            "eventType": "AwsApiCall",
            "recipientAccountId": "112233445566",
        },
    ),
    RuleTest(
        name="Terraform Security Group Authorize Egress",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROAIZCYMDBM4SJU65M54:ryan@example.com",
                "arn": "arn:aws:sts::112233445566:assumed-role/ContinousDeployment/ryan@example.com",
                "accountId": "112233445566",
                "accessKeyId": "ASIAWDWBPTM3RJH7XXXX",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROAIZCYMDBM4SJU65M54",
                        "arn": "arn:aws:iam::112233445566:role/Operator",
                        "accountId": "112233445566",
                        "userName": "Operator",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2020-04-30T23:50:12Z"},
                },
            },
            "eventTime": "2020-04-30T23:51:08Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "AuthorizeSecurityGroupEgress",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "100.1.114.142",
            "userAgent": "aws-sdk-go/1.29.7 (go1.13.7; darwin; amd64) APN/1.0 HashiCorp/1.0 Terraform/0.12.24 (+https://www.terraform.io)",
            "requestParameters": {
                "groupId": "sg-03eee825d6bb78f54",
                "ipPermissions": {
                    "items": [
                        {
                            "ipProtocol": "-1",
                            "groups": {},
                            "ipRanges": {
                                "items": [
                                    {
                                        "cidrIp": "0.0.0.0/0",
                                        "description": "Allow egress to the internet. Required for now until we land ECS/ECR endpoints in the VPC",
                                    },
                                ],
                            },
                            "ipv6Ranges": {},
                            "prefixListIds": {},
                        },
                    ],
                },
            },
            "responseElements": {"requestId": "4c7a5036-09d3-46e8-b0b6-f611ff1959a6", "_return": True},
            "requestID": "4c7a5036-09d3-46e8-b0b6-f611ff1959a6",
            "eventID": "1a593035-e072-49c4-8c72-4ff35e195330",
            "eventType": "AwsApiCall",
            "recipientAccountId": "112233445566",
        },
    ),
    RuleTest(
        name="Go Script Authorize Ingress",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROAIZCYMDBM4SJU65M54:ryan@example.com",
                "arn": "arn:aws:sts::112233445566:assumed-role/Operator/ryan@example.com",
                "accountId": "112233445566",
                "accessKeyId": "ASIAWDWBPTM3QPXQXXXX",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROAIZCYMDBM4SJU65M54",
                        "arn": "arn:aws:iam::112233445566:role/Operator",
                        "accountId": "112233445566",
                        "userName": "Operator",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2020-04-30T04:37:13Z"},
                },
            },
            "eventTime": "2020-04-30T04:40:52Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "AuthorizeSecurityGroupIngress",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "100.1.114.142",
            "userAgent": "aws-sdk-go/1.24.1 (go1.14; darwin; amd64)",
            "requestParameters": {
                "groupId": "sg-0d921df5d84f7270a",
                "ipPermissions": {
                    "items": [
                        {
                            "ipProtocol": "tcp",
                            "fromPort": 22,
                            "toPort": 22,
                            "groups": {},
                            "ipRanges": {"items": [{"cidrIp": "0.0.0.0/0"}]},
                            "ipv6Ranges": {},
                            "prefixListIds": {},
                        },
                    ],
                },
            },
            "responseElements": {"requestId": "2be70b99-4937-4a76-b7d9-390b6d0eda73", "_return": True},
            "requestID": "2be70b99-4937-4a76-b7d9-390b6d0eda73",
            "eventID": "42155429-7e8e-43b5-9b5e-6953f80d51d5",
            "eventType": "AwsApiCall",
            "recipientAccountId": "112233445566",
        },
    ),
    RuleTest(
        name="AWS Console - Ingress SG Authorization Error",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "UnauthorizedOperation",
            "eventID": "504b492f-7832-406b-a4fd-45a13e48adc4",
            "eventName": "AuthorizeSecurityGroupIngress",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2021-01-24 04:55:45.000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "112233445566",
            "requestID": "91f34d65-513d-4e9f-a3de-e8d27f7ee4b2",
            "requestParameters": {
                "groupId": "sg-04f0b44316f7d2471",
                "ipPermissions": {
                    "items": [
                        {
                            "ipRanges": {"items": [{"cidrIp": "0.0.0.0/16"}]},
                            "prefixListIds": {},
                            "fromPort": "443",
                            "toPort": "443",
                            "groups": {},
                            "ipProtocol": "tcp",
                            "ipv6Ranges": {},
                        },
                    ],
                },
            },
            "responseElements": {"_return": True, "requestId": "91f34d65-513d-4e9f-a3de-e8d27f7ee4b2"},
            "sourceIPAddress": "136.25.37.134",
            "userAgent": "console.ec2.amazonaws.com",
            "userIdentity": {
                "type": "AssumedRole",
                "principalid": "ARORJ4ULULLE0EEJAAKDO:alan",
                "arn": "arn:aws:sts::112233445566:assumed-role/TestAdmin/alan",
                "accountid": "112233445566",
                "accesskeyid": "ASIASWJRT64Z7ZLFLJNI",
                "sessioncontext": {
                    "attributes": {"mfaauthenticated": "true", "creationdate": "2021-01-24T04:55:10Z"},
                    "sessionissuer": {
                        "type": "Role",
                        "principalid": "ARORJ4ULULLE0EEJAAKDO",
                        "arn": "arn:aws:iam::112233445566:role/TestAdmin",
                        "accountid": "112233445566",
                        "username": "TestAdmin",
                    },
                },
            },
            "p_event_time": "2021-01-24 04:55:45.000",
            "p_parse_time": "2021-01-24 05:02:58.358",
            "p_log_type": "AWS.CloudTrail",
            "p_row_id": "1a57ff7ade26aaf5a1a4d7d20775",
            "p_source_id": "e55677c6-7ef5-4541-a443-0d0f17eec19f",
            "p_source_label": "CloudTrail Test",
            "p_any_ip_addresses": ["136.25.37.134"],
            "p_any_aws_account_ids": ["112233445566"],
            "p_any_aws_arns": [
                "arn:aws:iam::112233445566:role/TestAdmin",
                " arn:aws:sts::112233445566:assumed-role/TestAdmin/alan",
            ],
        },
    ),
]


@panther_managed
class AWSEC2ManualSecurityGroupChange(Rule):
    id = "AWS.EC2.ManualSecurityGroupChange-prototype"
    display_name = "AWS EC2 Manual Security Group Change"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    tags = ["AWS", "Security Control", "Configuration Required", "Defense Evasion:Impair Defenses"]
    default_severity = Severity.MEDIUM
    default_description = "An EC2 security group was manually updated without abiding by the organization's accepted processes. This rule expects organizations to either use the Console, CloudFormation, or Terraform, configurable in the rule's ALLOWED_USER_AGENTS.\n"
    default_runbook = "Identify the actor who changed the security group and validate it was legitimate"
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/working-with-security-groups.html"
    tests = awsec2_manual_security_group_change_tests
    PROD_ACCOUNT_IDS = {"11111111111111", "112233445566"}
    SG_CHANGE_EVENTS = {
        "CreateSecurityGroup": {
            "fields": ["groupName", "vpcId"],
            "title": "New security group [{groupName}] created by {actor}",
        },
        "AuthorizeSecurityGroupIngress": {
            "fields": ["groupId"],
            "title": "User {actor} has updated security group [{groupId}]",
        },
        "AuthorizeSecurityGroupEgress": {
            "fields": ["groupId"],
            "title": "User {actor} has updated security group [{groupId}]",
        },
    }
    # 'console.ec2.amazonaws.com',
    # 'cloudformation.amazonaws.com',
    ALLOWED_USER_AGENTS = {"* HashiCorp/?.0 Terraform/*"}
    ALLOWED_ROLE_NAMES = {"Operator", "ContinousDeployment"}

    def rule(self, event):
        # Validate the deployment mechanism (Console, CloudFormation, or Terraform)
        # Validate the IAM Role used is in our acceptable list
        return aws_cloudtrail_success(event) and (
            event.get("eventName") in self.SG_CHANGE_EVENTS.keys()
            and event.get("recipientAccountId") in self.PROD_ACCOUNT_IDS
            and (
                not (
                    pattern_match_list(event.get("userAgent"), self.ALLOWED_USER_AGENTS)
                    and any(role in deep_get(event, "userIdentity", "arn") for role in self.ALLOWED_ROLE_NAMES)
                )
            )
        )

    def dedup(self, event):
        return ":".join(
            deep_get(event, "requestParameters", field, default="<UNKNOWN_FIELD>")
            for field in self.SG_CHANGE_EVENTS[event.get("eventName")]["fields"]
        )

    def title(self, event):
        title_fields = {
            field: deep_get(event, "requestParameters", field, default="<UNKNOWN_FIELD>")
            for field in self.SG_CHANGE_EVENTS[event.get("eventName")]["fields"]
        }
        user = deep_get(event, "userIdentity", "arn", default="<UNKNOWN_USER>").split("/")[-1]
        title_template = self.SG_CHANGE_EVENTS[event.get("eventName")]["title"]
        title_fields["actor"] = user
        return title_template.format(**title_fields)

    def alert_context(self, event):
        return aws_rule_context(event)
